function LL = loglik_finmixMPEC_price_MAD_2R(theta)

% LOGLIK_FINMIXMPEC_PRICE_MAD_2R
% Log-Likelihood function for the Dynamic finite mixture random coefficients Logit.
% MPEC algorithm used with constraints:
%   1) Bellman's equation
%   2) Shares
%
% source: Dube, Fox and Su (2010)
% Code Revised: September 2010

global prods K Kp Kcov p X Xp Mkts T M R ooR sharesum denomexpand run iteration
global Chebyorder Chebypolyobs xiindex psiindex

% SET UP PARAMETERS
Theta = reshape(theta([1:K 1:2 K+1]),K,R);            % preferences
muparam = [theta(K+1+1:K+1+(R-1))];                   % mixing parameters
mu = [muparam;1-sum(muparam)];
thetap = theta(K+1+(R-1)+1:K+1+(R-1)+Kp);             % pricing function parameters
thetacov = theta(K+1+(R-1)+Kp+1:K+1+(R-1)+Kp+Kcov);

cholCOVu = zeros(2*prods,2*prods)*mu(1);
cholCOVu(1,1) = thetacov(1);
cholCOVu(2,2) = thetacov(1);
cholCOVu(3,3) = thetacov(3);
cholCOVu(4,4) = thetacov(3);
cholCOVu(1,3) = thetacov(2);
cholCOVu(2,4) = thetacov(2);
COVu = cholCOVu(1:end,1:end)'*cholCOVu(1:end,1:end);
a = COVu(1,1); b = COVu(1,2); c = COVu(1,3); d = COVu(1,4); 
e = COVu(2,1); f = COVu(2,2); g = COVu(2,3); h = COVu(2,4); 
i = COVu(3,1); j = COVu(3,2); k = COVu(3,3); l = COVu(3,4); 
m = COVu(4,1); n = COVu(4,2); o = COVu(4,3); pp = COVu(4,4);
detCOVu = i*c*h*n-i*c*f*pp+a*f*k*pp-a*h*k*n;
invCOVu = [ [-(k/(-a*k+c^2))	, 0 , (c/(-a*k+c^2))	, 0]
            [0	, -(k/(-a*k+c^2))	, 0 , (c/(-a*k+c^2))]
            [(c/(-a*k+c^2))	, 0 , -(a/(-a*k+c^2))	, 0]
            [0	, (c/(-a*k+c^2))	, 0 , -(a/(-a*k+c^2))]];

% EXPECTED VALUE OF WAITING AT OBSERVED STATES AND CHOICE-SPECIFIC VALUE FUNCTIONS
thetaChebyshev01 = theta(K+1+(R-1)+Kp+Kcov+1:K+1+(R-1)+Kp+Kcov+Chebyorder^2);
thetaChebyshev02 = theta(K+1+(R-1)+Kp+Kcov+1+Chebyorder^2:K+1+(R-1)+Kp+Kcov+2*Chebyorder^2);
V0obs1 = Chebypolyobs*thetaChebyshev01;
V0obs2 = Chebypolyobs*thetaChebyshev02;
xi = theta(K+1+(R-1)+Kp+Kcov+R*Chebyorder^2+1:K+1+(R-1)+Kp+Kcov+R*Chebyorder^2+prods*T*Mkts);	% demand shocks
Vjnetxi = X*Theta-repmat([V0obs1 V0obs2],prods,1);                                              % choice-specific value functions


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EVALUATE THE SHARES         %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
numer = exp(xi*ooR+Vjnetxi);            % this is the numerator (oo speeds-up expanding mean utility by number of types)
denom1 = sharesum*numer;
SS0 = 1./(1+denom1);                    % the outside share (or the inverse of the denominator)
Mt1 = M*(ones(T*Mkts,1)*mu(1));
Mt2 = M*(ones(T*Mkts,1)*mu(2));
for t=2:T,
    index = (0:Mkts-1)*T + t;
    Mt1(index,1) = SS0(index-1,1).*Mt1(index-1,1);    % mass of consumers at each mass point
    Mt2(index,1) = SS0(index-1,2).*Mt2(index-1,1);    % mass of consumers at each mass point
end
Mrt = [Mt1 Mt2]./((Mt1+Mt2)*ooR);       % time-specific mixing probabilities for each type "r"
denom = SS0(denomexpand,:);             % this expands the denominator
SS = numer.*denom;                      % type-specific shares

%%%%%%%%%%%%%%%%%%%
% PRICING PROCESS
%%%%%%%%%%%%%%%%%%%
phat = Xp*thetap;           % predicted prices at observed states
psi = p-phat;               % price shock


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EVALUATE THE LOG-LIKELIHOOD %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if max(max(isnan(xi))) == 1,
	LL = 1e+10+Theta(1);
else
    densityconstant = (2*pi)^(-2*prods/2)*(detCOVu)^(-.5);
    aa = invCOVu(1,1); bb = invCOVu(1,2); cc = invCOVu(1,3); dd = invCOVu(1,4);
    ee = invCOVu(2,1); ff = invCOVu(2,2); gg = invCOVu(2,3); hh = invCOVu(2,4); 
    ii = invCOVu(3,1); jj = invCOVu(3,2); kk = invCOVu(3,3); ll = invCOVu(3,4); 
    mm = invCOVu(4,1); nn = invCOVu(4,2); oo = invCOVu(4,3); ppp = invCOVu(4,4);
    % the following expression for the density is used to faciliate automatic differentiation
    phi = densityconstant*exp( -.5*(xi(xiindex(:,1)).*(cc*psi(psiindex(:,1))+kk*xi(xiindex(:,1)))+psi(psiindex(:,2)).*(ff*psi(psiindex(:,2))+nn*xi(xiindex(:,2)))+xi(xiindex(:,2)).*(hh*psi(psiindex(:,2))+ppp*xi(xiindex(:,2)))+psi(psiindex(:,1)).*(ii*xi(xiindex(:,1))+aa*psi(psiindex(:,1)))) );
    phi(phi==0) = 1.e-300;
    logdetJacob = 0;
    for tt=1:T*Mkts,
        index = (0:prods-1)*T*Mkts+tt;
        SSS = SS(index,1);
        Jacob = (diag(SSS)-SSS*SSS')*Mrt(tt,1);                   % Jacobian
        for rr = 2:R,
            SSS = SS(index,rr);
            Jacob = Jacob + (diag(SSS)-SSS*SSS')*Mrt(tt,rr);
        end
        logdetJacob = logdetJacob + log(1/det(Jacob));
    end
	LL = -sum(log(phi)) - logdetJacob;
    if sum(isinf(getderivs(LL)))>0,
        LL = 1.e+10+Theta(1);
    end
end