function [Shares,S,pricet,lagpricet,Mt,market,period,uttrue,thetaChebyshev0] = mkdata_dynamicNFP_inerp(prodsset,beta,lambda,sigma,OMEGA,M,Tcap,N,p0,thetaChebyshev)

%%%%%%%%%%%%%%%%%%%%%%%
% MKDATA_DYNAMICNFP_Cheby
% 
% Solve consumer DP and then simulate datasets for the dynamic BLP model
% i.e. r.c. Logit demand with finite-mixture distribution of heterogeneity.
%  Consumer's value function approximated on a grid with Chebyshev Polynomials.
%
% 1)  prodsset = # products
% 2)  beta = taste parameters (separate column for each consumer type)
% 3)  lambda = parameters for mixing probabilities (i.e. 1 less than total # types)
% 4)  sigma = parameters of pricing function
% 5)  OMEGA = covariance matrix of price and demand shocks
% 6)  deltaset = discount factor
% 7)  M = mass of consumers in population
% 8)  Tcap = length of simulated history
% 9)  N = # simulated markets
% 10) p0 = initial prices in each market
% 11) V00 = optional consumer value function (i.e. if want to by-pass solving DP by supplying a solution)
%
% source: Dube, Fox and Su (2008)
% Code Revised: May 2008
%
% \xi(j,t) ~ i.i.d.N(0,sigma2xi)
% price(j,t) = sigma0_j + sigma1_j*price(1,t-1) + sigma2_j*price(2,t-1) + \xi(j,t)
% U(j,t,r) = betaj_r + beta_r*price(j,t) + \epsilon(j,t) + \xi(j,t)
% U(0,t,r) = epsilon(0,t)
%
%%%%%%%%%%%%%%%%%%%%%%%

global oo quadpts
global prods R qu thetaChebyshev0 Chebyorder a1 a2 b1 b2 oo1 oo2               % indices for Chebyshev approximation

R = size(beta,2);                               % number of consumer types
prods = prodsset;

if size(p0,1)~=N, disp('initial prices have wrong shape'); STOP; end
if size(beta,2)~=R || size(beta,1)<prods, disp('taste parameters have wrong shape'); STOP; end;
if length(sigma)~=prods*(prods+1), disp('length of pricing function parameters is wrong'); STOP;end;
if size(OMEGA)~=2*prods, disp('dimension of covariance matrix of shocks is wrong'); STOP; end;

if nargin<10,
    %%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% CONSUMER'S DP %%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%
    qu = sqrt(2)*quadpts*chol(OMEGA);           % adjust quadrature points for the co-variance of the distribution of \psi and \xi
    if prods==1,
        thetaChebyshev = valuefunctions_Chebyshev1dim(beta,sigma,qu);
    elseif prods==2,
        thetaChebyshev = valuefunctions_Chebyshev2dim(beta,sigma,qu);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% SIMULATE A HISTORY %%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if R==1,
    lambda=1;
else
    lambda = [lambda 1-sum(lambda)];
end
uttrue = zeros(Tcap*N,2*prods);
Shares = zeros(Tcap*N,prods+1,R);
S = zeros(Tcap*N,prods+1);
Mt = zeros(Tcap*N,R);
pricet = zeros(Tcap*N,prods);
lagpricet = zeros(Tcap*N,prods);
Mt( (0:N-1)*Tcap+1 ,:,:) = repmat(lambda*M,N,1);    % initial Market size in each market
Qcum = zeros(Tcap*N,prods);
y = zeros(1,prods);
market = kron( (1:N)',ones(Tcap,1));
period = repmat( (1:Tcap)',N,1);

lagpricet((0:N-1)*Tcap+1 ,:) = p0;                                % initial prices
for ii =1:N,
    ut = randn(Tcap,2*prods)*chol(OMEGA);
    uttrue( (ii-1)*Tcap+1:ii*Tcap,:) = ut;
    for tt = 1:Tcap,
        index = (ii-1)*Tcap + tt;
        priceii = kron(eye(prods),[1 lagpricet(index,:)])*sigma + ut(tt,1:prods)';
%        priceii = pricet(index,:);
        st = priceii';                              % current price is next-period state
        if prods==1,
            rhox1 = ones(1,Chebyorder);
            rhox1(:,2) = 2*(st(:,1)-a1)/(b1-a1)-1;      % transformed state variable 1 next period
            for zz=3:Chebyorder,
                rhox1(:,zz) = 2*rhox1(:,2).*rhox1(:,zz-1) - rhox1(:,zz-2);      %note used 2nd column of rho to obtain transformed state variable
            end
            Xfuture = rhox1;       % tensor product of transformed state variables next period            
        elseif prods==2,
            rhox1 = ones(1,Chebyorder);
            rhox2 = rhox1;
            rhox1(:,2) = 2*(st(:,1)-a1)/(b1-a1)-1;      % transformed state variable 1 next period
            rhox2(:,2) = 2*(st(:,2)-a2)/(b2-a2)-1;      % transformed state variable 2 next period
            for zz=3:Chebyorder,
                rhox1(:,zz) = 2*rhox1(:,2).*rhox1(:,zz-1) - rhox1(:,zz-2);      %note used 2nd column of rho to obtain transformed state variable
                rhox2(:,zz) = 2*rhox2(:,2).*rhox2(:,zz-1) - rhox2(:,zz-2);
            end
            Xfuture = rhox1(:,oo1).*rhox2(:,oo2);       % tensor product of transformed state variables next period
        end

        V0 = Xfuture*thetaChebyshev;
        uu = [[oo priceii]*beta + repmat(ut(tt,prods+1:2*prods)',1,R) ; V0];
        numer = exp(uu);
        denom = sum(numer);
        sh = numer./repmat(denom,prods+1,1);
        QQ = sum(sh(1:prods,:).*repmat(squeeze(Mt(index,:)),prods,1),2)';
        Qcum(index,:) = y + QQ;
        y = Qcum(index,:);
        Shares(index,:,:) = sh;
        lambdatt = ones(prods+1,1)*Mt(index,:)./sum(Mt(index,:),2);
        S(index,:) = sum(sh.*lambdatt,2);
        pricet(index,:) = priceii;
        if tt<Tcap,
            Mt(index+1,:) = sh(prods+1,:).*Mt(index,:);
            lagpricet(index+1,:) = priceii;
        end
    end
end
