%%%%%%%%%%%%%%%%%%%%%%%
% DYNAMIC BLP with Random Coefficient only on Price
%
% Create Dynamic BLP data and then estimate the model using NFP
%
% NOTE: You can pre-set the data simulation to construct MANY markets and
% MANY time periods.  This way we can run the data construction once and
% still use it to experiment with (1) more markets and (2) longer histories
% without having to re-run things over and over.  So, you probably want
% "Tcap" and "N" to be big.  You also probably want a lot of variation in
% "p0"
%
% source: Dube, Fox and Su (2010)
% Code Revised: Sept 2010
%
% \xi(j,t) ~ i.i.d.N(0,sigma2xi)
% price(j,t) = sigma0_j + sigma1_j*price(1,t-1) + sigma2_j*price(2,t-1) + \xi(j,t)
% U(j,t,r) = betaj_r + beta_r*price(j,t) + \epsilon(j,t) + \xi(j,t)
% U(0,t,r) = epsilon(0,t)
%
%%%%%%%%%%%%%%%%%%%%%%%
global tolV report states oo oostates ooR quadpts qwts sims run iteration report M R delta T Mkts
global  ProjChebpoly Chebpoly oo1 oo2 Chebygrid a1 a2 b1 b2 thetaChebyshev0 Chebygrid Chebygridpts ooChebygrid Chebyorder

diary NFPMAD.out;

randn('seed',1)
rand('seed',1)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% SETTINGS FOR THE EXPERIMENT %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
prods = 2;                                          % Number of products available
betatrue = [ [4; 3; -1] , [4;3;-2] ];               % true preferences
R = size(betatrue,2);                               % number of consumer types
lambdatrue = .7;                                    % true mixing probabilities over types
sigmatrue = [5 .8 .2 5 .1 .55]';                    % true parameters of pricing strategy functions
sigmatrue = [.2 .8 0 .2 0 .8]';
OMEGAtrue = kron(ones(2),eye(prods))/2 + kron(eye(2),eye(prods))/2;                % covariance matrix of \xi and price shocks
cholOMEGAtrue = chol(OMEGAtrue);
delta = 0.99;                                       % consumer's discount factor
M = 1;                                              % Mass of potential consumers at date 0
report =20;                                         % frequency at which to report estimates during search
tolV = 1.e-14;                                      % convergence tolerance for consumer DP value-function iteration
Tcap = 50;                                          % length of simulated data produced (i.e. # periods)
N = 20*5;                                           % number of markets (will determine number Monte Carlo experiments!)
p0 = repmat([10,10],N,1) + [rand(N,1)*5 rand(N,1)*5];         % initial prices

%%%%%%%%%%%%%%%%
% SETTINGS FOR THE DATASET TO EXTRACT FOR EACH MONTE CARLO
%%%%%%%%%%%%%%%%
Mkts = 5;                                               % number of simulated markets used for estimation
T = Tcap;                                                 % number of periods retained from each market

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Quadrature Points for a Multivariate Standard Normal %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
qpts = 3;                                                   % number of quadrature points to use
[quadpts,quadwts] = gauher_multidim(2*prods,qpts);          % create quadrature nodes and weights for integration of \xi states
qwts = quadwts/sqrt(pi)^(2*prods);
sims = length(qwts);                                        % total number of grid points spanned by quadrature grid
states = qpts^(2*prods);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% STATE GRID (current prices)        %%%
%%%% Chebyshev Polynomials
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Chebynodes = 5;
Chebyorder = 4;
upperboundprice = max(max(p0))*2;
a1=0;
a2 = a1;
b1 = upperboundprice;
b2 = b1;
[ProjChebpoly,Chebpoly,oo1,oo2,Chebygrid] = Chebyshevsetup2dim(Chebynodes,Chebyorder,a1,b1,a2,b2);
thetaChebyshev0 = ProjChebpoly*zeros(Chebynodes^2,R);
Chebygridpts = Chebynodes^2;
ooChebygrid = ones(Chebygridpts,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 1) Simulate the Data                                    %%%
%%% NOTE: unlike static models our data are now sorted by   %%%
%%%      by market then product (not vice-versa).           %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
oo = eye(prods);
oostates = ones(states,1);
ooR = ones(1,R);
randn('seed',155)                                   % reset normal random number generator
rand('seed',155)                                    % reset uniform random number generator

[Sharessim,Ssim,pricessim,lagpricessim,Mtsim,marketsim,periodsim,utsim,thetaChebyshevsim] = mkdata_dynamicNFP_Cheby(prods,betatrue,lambdatrue,sigmatrue,OMEGAtrue,M,Tcap,N,p0);
%save(['dynamicdata_Cheby_price_' num2str(delta*100) 'delta_' num2str(R) 'types'],'betatrue','sigmatrue','lambdatrue','prods','OMEGAtrue','delta','M','Tcap','N','p0','Sharessim','Ssim','pricessim','lagpricessim','Mtsim','marketsim','periodsim','utsim','thetaChebyshevsim')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 2) Estimate the model       %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global Sh Sh0 P p meanval X Xp stateobs K Kp Kcov setupcov setupthetacov expxi0 denomexpand sharesum tol_inner Chebypolyobs
global Xstate1 Xstate2 quadpts_estim xiindex psiindex

R = size(betatrue,2);
obs = prods*T*Mkts;
setupcov = find(chol(OMEGAtrue));                       % impose correct covariance restrictions
setupthetacov = [1 1 3 2 3 2];                          % elements of theta matrix
Kcov = 3;                                               % number of covariance terms to estimate
sharesum = kron( ones(1,prods),speye(T*Mkts));          % used to create denominators in shares (sums numerators)
denomexpand = repmat( (1:T*Mkts)',prods,1);             % used to expand denominator in shares

thetaNFP = [];
GNFP = [];
CPUtNFP = [];
FuncEvalNFP = [];
INFONFP = [];
GradEvalNFP = [];
runNFP = [];

%%% ITERATE OVER DATASETS
for rr=1:floor(N/Mkts),
    %%% Open data
    keepindex = repmat((1:T)',Mkts,1) + kron( (rr-1)*Mkts*Tcap+(0:Mkts-1)'*Tcap, ones(T,1));
    Sh = reshape(Ssim(keepindex,1:prods,:),obs,1);      % market shares
    Sh0 = repmat(Ssim(keepindex,prods+1),prods,1);      % outside share
    P = reshape(pricessim(keepindex,:),obs,1);          % prices
    lagP = reshape(lagpricessim(keepindex,:),obs,1);    % lag prices
    p = P;                                              % price configuration used as dependent variable
    psitrue = reshape(utsim(keepindex,1:2),obs,1);      % true shocks
    xitrue  = reshape(utsim(keepindex,prods+1:2*prods),obs,1);
    Mt = Mtsim(keepindex,:);
    Xp = kron(eye(prods),[ones(T*Mkts,1) lagpricessim(keepindex,:)]);      % sort matrix by market then product
    FE = kron(eye(prods),ones(T*Mkts,1));
    X = [ FE P ];
    IV = [FE lagP];
    stateobs = reshape(P,T*Mkts,prods);                 % observed states
    meanval = log(Sh) - log(Sh0);
    K = size(X,2);                                      % number demand parameters
    Kp = size(Xp,2);                                    % number price parameters

    %%%%%%%%%%%%%%%%%%%%%%%%%
    % OLS and 2sls estimates
    %%%%%%%%%%%%%%%%%%%%%%%%%
    Xhat = [full(X(:,1:end-1)) IV*inv(IV'*IV)*IV'*P];
    beta2sls = inv(Xhat'*Xhat)*Xhat'*meanval;
    xi2sls = meanval - X*beta2sls;
    se2sls = sqrt(diag( xi2sls'*xi2sls*inv(Xhat'*Xhat) ));
    t2sls = beta2sls./se2sls;
    betaolsp = inv(Xp'*Xp)*Xp'*p;

    %%%%%%%%%%%%%%%%%%%%%%%%%
    % Set Up Chebyshev grid for estimation
    %%%%%%%%%%%%%%%%%%%%%%%%%
    Chebynodes = 5;
    Chebyorder = 4;
    upperboundprice = max(max(p0))*2;
    a1=0;
    a2 = a1;
    b1 = upperboundprice;
    b2 = b1;
    [ProjChebpoly,Chebpoly,oo1,oo2,Chebygrid,Chebypolyobs] = Chebyshevsetup2dim(Chebynodes,Chebyorder,a1,b1,a2,b2,stateobs);
    Chebygridpts = Chebynodes^2;
    ooChebygrid = ones(Chebygridpts,1);

    % for use in value function iteration
    Xstate = kron(oo,[ooChebygrid Chebygrid(:,1:prods)]);
    Xstate1 = repmat(Xstate(1:Chebygridpts,:),sims,1);
    Xstate2 = repmat(Xstate(Chebygridpts+1:end,:),sims,1);
    quadpts_estim = kron(quadpts,ooChebygrid);
    xiindex = [(1:T*Mkts)' (T*Mkts+1:2*T*Mkts)'];
    psiindex = [(1:T*Mkts)' (T*Mkts+1:2*T*Mkts)'];    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% START VALUES
    ooR = ones(1,R);
    thetatrue = [betatrue(:,1);betatrue(end,2);lambdatrue;sigmatrue;cholOMEGAtrue([1 11 9])'; reshape(thetaChebyshevsim,R*Chebyorder^2,1); xitrue];
    %Kcov = prod(size(chol(OMEGAtrue)));
    Kcov = 3;
    theta0 = zeros(K+1+(R-1)+Kp+Kcov,1);
    theta0(1:K+1) = beta2sls([1:K K]);                    % Use 2sls estimates
    theta0(K+1) = theta0(K+1)*1.01;                         % add noise to price parameter in 2nd segment
    theta0(K+1+1:K+1+(R-1)) = 0.55;
    theta0(K+1+(R-1)+1:K+1+(R-1)+Kp) = betaolsp;
    u0 = [p-Xp*betaolsp meanval-X*beta2sls];
    CCu0 = cov(u0);
    COVu0 = zeros(2*prods);
    COVu0(1,1) = CCu0(1,1);
    COVu0(2,2) = CCu0(1,1);
    COVu0(3,3) = CCu0(2,2);
    COVu0(4,4) = CCu0(2,2);
    COVu0(3,1) = CCu0(1,2);
    COVu0(4,2) = CCu0(1,2);
    COVu0(1,3) = CCu0(1,2);
    COVu0(2,4) = CCu0(1,2);
    cholCOVu0 = chol(COVu0);
    thetacov0 = cholCOVu0([1 9 11]);
    theta0(end-Kcov+1:end) = thetacov0;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% NFP Approach via Chebyshev%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    theta0_list  = repmat(theta0',5,1) .* cat(2,ones(size(theta0)),1+0.1*randn([4,size(theta0',2)] )'./repmat(abs(theta0),1,4) )';
	theta0_list  = repmat(theta0',5,1) .* cat(2,ones(size(theta0)),1+0.1*randn([4,size(theta0',2)] )' )';

    CPUt = 0;
    FuncEval = 0;
    GradEval = 0;
    run=1;
    G=1.e+10;
    report = 5000;    
    INFO=-1000;
    %%% ITERATE OVER START VALUES
    for reps = 1:5,
%    for reps = 1,
        % START VALUES
        theta00 = theta0_list(reps,:)' ;
        Theta = reshape(theta00([1:K 1:2 K+1]),K,R);
        thetap = theta00(K+1+(R-1)+1:K+1+(R-1)+Kp);
        thetacov = theta00(K+1+(R-1)+Kp+1:K+1+(R-1)+Kp+Kcov);
        cholCOVu = zeros(2*prods,2*prods);
        cholCOVu(1,1) = thetacov(1);
        cholCOVu(2,2) = thetacov(1);
        cholCOVu(3,3) = thetacov(3);
        cholCOVu(4,4) = thetacov(3);
        cholCOVu(1,3) = thetacov(2);
        cholCOVu(2,4) = thetacov(2);
        COVu = cholCOVu(1:end,1:end,1)'*cholCOVu(1:end,1:end,1);
        a = COVu(1,1); b = COVu(1,2); c = COVu(1,3); d = COVu(1,4); 
        e = COVu(2,1); f = COVu(2,2); g = COVu(2,3); h = COVu(2,4); 
        i = COVu(3,1); j = COVu(3,2); k = COVu(3,3); l = COVu(3,4); 
        m = COVu(4,1); n = COVu(4,2); o = COVu(4,3); pp = COVu(4,4);
        detCOVu = i*c*h*n-i*c*f*pp+a*f*k*pp-a*h*k*n;
        invCOVu = [ [-(k/(-a*k+c^2))	, 0 , (c/(-a*k+c^2))	, 0]
            [0	, -(k/(-a*k+c^2))	, 0 , (c/(-a*k+c^2))]
            [(c/(-a*k+c^2))	, 0 , -(a/(-a*k+c^2))	, 0]
            [0	, (c/(-a*k+c^2))	, 0 , -(a/(-a*k+c^2))]];
        qu = sqrt(2)*quadpts_estim*cholCOVu(1:end,1:end,1);    % quadrature points to integrate over the distribution of shocks
        thetaChebyshev0 = zeros(size(Chebpoly,2),R);
        thetaChebyshev0 = valuefunctions_Chebyshev2dim_estim_noMAD(Theta,thetap,qu);
        V0obs = Chebypolyobs*thetaChebyshev0;                % expected value of waiting
        expVjnetxi0 = exp(X*Theta-repmat(V0obs,prods,1));    % exponentiated choice-specific value functions        
        tol_inner = 1.e-14;
        expxi0 = exp(xi2sls);
%        xi0 = invertnfp_finmix_noMAD(expVjnetxi0,[theta00(K+1+(R-1)); 1-theta00(K+1+(R-1))]);
%        expxi0 = exp(xi0);

        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% Create estimation bounds
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        fLowBnd = -Inf;                                     % Lower bound on function.
        thetalow = zeros(K+1+(R-1)+Kp+Kcov,1);
        thetalow(1:end) = -Inf;                             % tastes, \xi and Chebyshev parameters are unbounded below
        thetalow(1:length(theta0)) = -10;
        thetalow(K+1+(R-1)) = 0;                            % mixing probability lies above 0 (only for R>1)
        thetaupp = thetalow*0;
        thetaupp(1:end) = Inf;
        thetaupp(1:length(theta0)) = 10;
        thetaupp(K+1+1:K+1+(R-1)) = 1;                      % mixing probability lies below 1
        thetaupp( [K K+1] ) = 0;                            % constrain price parameter to be negative
        Name = 'Fin Mix NFP MLE Problem via Chebyshev';
        
        tol_inner = 1.e-14;
        tol_outer = 1.e-6;
        tolV = 1.e-14;
        iteration=1;
        
        Prob = conAssign('loglik_finmixNFP_price_2R', [], [], [], thetalow, thetaupp, Name, theta00,...
            [], fLowBnd, [], [], [], [], [], [], [], [], []);
        Prob.PriLevOpt = 3;
        Prob.KNITRO.options.ALG = 2;                    % Select feasible 
        Prob.KNITRO.PrintFile = ''; 
        Prob.KNITRO.options.MAXIT = 5000;               % Setting maximum number of iterations     
        Prob.KNITRO.options.OPTTOL = tol_outer ;        % set outer-loop tolerance       
        Prob.KNITRO.options.HONORBNDS = 1 ;             % ensure parameter search remains within constraints

        %%Automatic differentiation
        % COMMENT THESE LINES OUT TO SHUT OFF AUTOMATIC DIFF
        madinitglobals;     % Initiate MAD variables
        Prob.ADObj = 1;     % replace 1 by -1 if using second-order derivatives
        Prob.ADCons = 1;    % replace 1 by -1 if using second-order derivatives

        Result = tomRun('knitro', Prob, 1);

        CPUt = CPUt + Result.CPUtime;
        FuncEval = FuncEval + Result.FuncEv;
        GradEval = GradEval + Result.GradEv;
        
        if reps==1,
            theta = Result.x_k;
            theta = Result.x_k;
            G = Result.f_k;
            INFO = Result.Inform;        
        end        
        if (Result.f_k < G && Result.Inform==0),
            theta = Result.x_k;
            G = Result.f_k;
            INFO = Result.Inform;
        end
    end

    thetaNFP = [thetaNFP,theta];
    GNFP = [GNFP,G];
    CPUtNFP = [CPUtNFP,CPUt];
    FuncEvalNFP = [FuncEvalNFP,FuncEval];
    INFONFP = [INFONFP,INFO];
    GradEvalNFP = [GradEvalNFP,GradEval];
    runNFP = [runNFP,run];
    save tempNFP_MAD_2R thetaNFP GNFP INFONFP CPUtNFP FuncEvalNFP GradEvalNFP theta0 runNFP
    
end
save(['MCestimatesNFP_price_MAD_' num2str(R) 'R'], 'thetaNFP', 'GNFP', 'INFONFP', 'CPUtNFP', 'FuncEvalNFP', 'GradEvalNFP', 'theta0', 'runNFP', 'N');