%%%%%%%%%%%%%%%%%%%%%%%
% MAIN SCRIPT
%
% 1) Simulate Data from Random Coefficients Logit Demand model
% 2) Estimate model using MPEC
%
% source: Dube, Fox and Su (2009)
% Code Revised: May 2009
%%%%%%%%%%%%%%%%%%%%%%%

%%% This is the MATLAB script file to run Monte Carlo experiments with large number of markets (reported
%%% in Table 5 and 6) in Dube, Fox and Su (2011). 
%%% In this version, I sort market first and then product. 
%%% This sorting scheme gives a block diagonal structure in consraints Jacobian
%%% I also use sparse matrices to represent constraint Jacobian, the
%%% Hessian and their sparsity patterns. 
%%% 
%%% January, 2011

% profile on

%%%%%%%%%%% 
% SETTINGS
%%%%%%%%%%%
global share W PX x IV rc expmeanval nIV % logshare
global datasort denomexpand sharesum oo
global T prods nn K v rc rctrue oo oo1 tol_inner tol_outer ConsPattern d2LPatt

% aa=maxNumCompThreads(1);                            % disable multi-thread processing b/c Knitro cannot exploit this so it slows the code down
% This command is obslete. Use the command to start Matlab session in X11:
% matlab -singleCompThread


diary output_MPEC_T100_prods25_nn3000_sparse.out;

randn('seed',155)                                   % reset normal random number generator
rand('seed',155)                                    % reset uniform random number generator
nn = 100;                                           % # draws to simulate shares
tol_inner = 1.e-14;
tol_outer = 1.e-6;
prods = 25;                                         % # products
T = 100;                                             % # Markets
starts = 5;                                         % # random start values to check during estimation

disp(['This Monte Carlo experiment is for ' num2str(T) ' markets, ' num2str(prods) ' products, ' num2str(nn) ' draws with ' num2str(starts) ' restarts.'])

%%%%%%%%%%%%%%%%%%%%%%%%
% TRUE PARAMETER VALUES
%%%%%%%%%%%%%%%%%%%%%%%%
costparams = ones(6,1)/2;
betatrue = [2 1.5 1.5 .5 -3]';                     % true mean tastes
betatrue0 = betatrue;
K = size(betatrue,1)-1;                             % # attributes
covrc = diag( [ .5 .5 .5 .5 .2] );                  % true co-variances in tastes
rctrue = sqrt(covrc(find(covrc)));
thetatrue = [betatrue;rctrue];
v = randn(length(betatrue),nn);                     % draws for share integrals during estimation
rc = chol(covrc)'*v;                                % draws for share integrals for data creation
sigmaxi=1;
covX = -.8*ones(K-1)+1.8*eye(K-1);                  % allow for correlation in X's
covX(1,3) = .3;
covX(2,3) = .3;
covX(3,1) = .3;
covX(3,2) = .3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Create indices to speed share calculations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
oo = ones(1,nn);                                    % index for use in simulation of shares
oo1 = (0:T-1)*prods+prods;                          % indicates last observation for each market
sharesum = kron(speye(T), ones(1,prods));           % used to create denominators in predicted shares (i.e. sums numerators)
datasort = [(1:T*prods)' repmat((1:T)',prods,1) kron( (1:prods)',ones(T,1))];
datasort = datasort(:,1);
denomexpand = kron((1:T)',ones(prods,1));

%%%%%%%%%%%%%%%%%%%%%
% SIMULATE DATA
%%%%%%%%%%%%%%%%%%%%%
randn('seed',5000)                                   % reset normal random number generator
rand('seed',5000)                                    % reset uniform random number generator
% randn('seed',500)                                   % reset normal random number generator
% rand('seed',500)                                    % reset uniform random number generator
xi = randn( T*prods,1)*sigmaxi;                      % draw demand shocks
A = [kron(ones(T,1), randn( prods,K-1)*chol(covX))];  % product attributes
prand = rand( T*prods,1)*5;
price = 3 +   xi*1.5 +  prand + sum(A,2);
z = rand( T*prods,length(costparams)) + 1/4*repmat( abs( prand +  sum(A,2)*1.1 ) ,1,length(costparams));
x = [ones(T*prods,1) A price];
[share,nopurch] = mksharesim(betatrue,x,xi,rc);
y = log(share) - log(kron(nopurch,ones(prods,1)));      % log-odds ratio
iv = [ones(prods*T,1) A z];                         % matrix of instruments

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2SLS ESTIMATION OF HOMOGENEOUS LOGIT MODEL %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('2SLS Estimation of Homogenous Logit Model')
% xhat = iv*myinv(iv'*iv)*(iv'*x);
% PX2sls = myinv(xhat'*xhat)*xhat';                     % project. matrix on weighted x-space for 2SLS
xhat = iv*((iv'*iv)\(iv'*x));
PX2sls = (xhat'*xhat)\(xhat');                     % project. matrix on weighted x-space for 2SLS
beta2sls = PX2sls*y;
% se2sls = sqrt(diag( mean((y-x*beta2sls).^2)*myinv(xhat'*xhat) ));
nxhat = size(xhat,2);
se2sls = sqrt(diag( mean((y-x*beta2sls).^2)*((xhat'*xhat)\eye(nxhat)) ));

expmeanval0 = exp(y);
IV = [ones(T*prods,1) A z A.^2 A.^3 z.^2 z.^3 prod(A,2) prod(z,2) kron(A(:,1),ones(1,size(z,2))).*z  kron(A(:,2),ones(1,size(z,2))).*z];
% W = inv(IV'*IV);
% PX = inv(x'*IV*W*IV'*x)*x'*IV*W*IV';
nIV = size(IV,2);           % # instrumental variables
W = (IV'*IV)\eye(nIV);
PX = ((x'*IV)*W*(IV'*x))\(x'*IV*W*IV');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MPEC ESTIMATION OF RANDOM COEFFICIENTS LOGIT MODEL %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
theta20 = 0.5*abs(beta2sls);
startvalues = repmat(theta20',starts,1).* cat(2,ones(size(theta20)),rand([starts-1,size(theta20',2)] )' *1 )';

GMPEC = 1.0e20;
CPUtMPEC = 0;
FuncEvalMPEC = 0;
GradEvalMPEC = 0;
expmeanval = expmeanval0;
 
for reps=1:starts,
    
    %%% NOTE: START VALUES SET TO GIVE SAME INITIAL GMM OBJECTIVE FUNCTIONAL
    %%% VALUE AS WITH NFP
    
    disp('Constructing starting values')
    
    t1 = cputime;
    theta20 = startvalues(reps,:)';
    % delta0 = 2*ones(T*prods,1);
    delta0 = invertshares(theta20);
    t1 = cputime-t1;
    disp('End of Constructing starting values')
    disp(['time spent on constructing starting point: ' num2str(t1) ' sec.'])
    theta10 = PX*delta0;
    resid0 = delta0 - x*theta10;
    g0 = IV'*resid0;
    x0 = [theta10; theta20; delta0; g0];
    
    disp('Calling runGMMMPECTomlab')
    
    [theta1MPEC_rep,theta2MPEC_rep,GMPEC_rep,INFOMPEC_rep,CPUtMPEC_rep,FuncEvalMPEC_rep,GradEvalMPEC_rep, d2L, lambda, MPECx] = runGMMMPECTomlab(x0,1);
    
    CPUtMPEC = CPUtMPEC + CPUtMPEC_rep;
    FuncEvalMPEC = FuncEvalMPEC + FuncEvalMPEC_rep;
    GradEvalMPEC = GradEvalMPEC + GradEvalMPEC_rep;
    
    if (GMPEC_rep < GMPEC && INFOMPEC_rep==0),
        theta1MPEC = theta1MPEC_rep;
        theta2MPEC = theta2MPEC_rep;
        GMPEC = GMPEC_rep;
        INFOMPEC = INFOMPEC_rep;
    end
    
    
end

% COVARIANCE MATRIX FOR MPEC STRUCTURAL PARAMETERS
% delta = invertshares(thetaMPEC2);    % mean utilities
% resid = delta - x*thetaMPEC1;  % xi
% Ddelta = jacobian(thetaMPEC2);       % jacobian matrix of mean utilities
% covg = zeros(size(IV,2));
% for ii =1:length(IV),
%     covg = covg + IV(ii,:)'*IV(ii,:)*(resid(ii)^2);
% end
% Dg = [x Ddelta]'*IV;            % gradients of moment conditions
% covMPEC = inv( Dg*W*Dg')*Dg*W*covg*W*Dg'*inv( Dg*W*Dg');

save(['MPEC_Hess_' num2str(T) 'mkts_' num2str(prods) 'products',num2str(nn),'draws'],'theta1MPEC', 'theta2MPEC','thetatrue', 'GMPEC', 'CPUtMPEC', 'FuncEvalMPEC', 'GradEvalMPEC', 'INFOMPEC');        
        
% profile off
% profview

diary off;